/// <reference types="node" />

import { Context, ObjectId, Resolvable, Sequence, StepFunction } from '@agentframework/app';
import { IProfile, Model, PrimaryKey } from '@agentframework/data';
import { ClassInitializer } from '@agentframework/domain';
import { WebApplication, WebApplicationSettings, WebContext, WebRequest, WebRequestIdentity } from '@agentframework/web';

export interface EmailProvider {
	provider: string;
	domain: Array<string>;
}
export declare class WebFoundationSettings extends WebApplicationSettings {
	PUBLIC_DIR: string;
	PUBLIC_DIR_CACHE_EXPIRES_IN_SECONDS: number;
	LOG_SLOW_REQUEST_IN_MILLISECONDS: number;
	WEB_DEVICE_COOKIE_DOMAIN: string;
	WEB_DEVICE_COOKIE_KEY: string;
	WEB_DEVICE_COOKIE_EXPIRES_IN_SECONDS: number;
	WEB_SESSION_COOKIE_KEY: string;
	WEB_SESSION_COOKIE_EXPIRES_IN_SECONDS: number;
	WEB_SESSION_EXPIRES_IN_MILLISECONDS: number;
	WEB_CONTENT_SECURITY_POLICY: Array<string>;
	ADMIN_EMAIL: string;
	TRUSTED_EMAIL_PROVIDERS: Array<EmailProvider>;
	THEME: string;
	OWNER_NAME: string;
	OWNER_COMPANY_NAME: string;
	OWNER_COMPANY_CODE: string;
	OWNER_COMPANY_ADDRESS: Array<string>;
	OWNER_COMPANY_TELEPHONE: string;
	OWNER_COMPANY_FAX: string;
	OWNER_AGREEMENTS: Array<any>;
	WEB_TOKEN_EXPIRES_IN_MILLISECONDS: number;
	WEB_INVITATION_EXPIRES_IN_MILLISECONDS: number;
	WEB_EMAIL_VERIFICATION_RESEND_COOLDOWN_IN_MILLISECONDS: number;
	WEB_EMAIL_VERIFICATION_EXPIRES_IN_MILLISECONDS: number;
	WEB_SIGNIN_PATH: string;
	WEB_SIGNIN_SUCCESS_PATH: string;
	WEB_SIGNIN_FAILED_PATH: string;
	WEB_ACCESS_DENIED_PATH: string;
	WEB_SIGNOUT_PATH: string;
	WEB_SIGNOUT_COMPLETE_PATH: string;
	OIDC_AUTHENTICATION_EXPIRES_IN_MILLISECONDS: number;
	OIDC_REDIRECT_LIST: Array<string>;
	OIDC_NO_REDIRECTS_LIST: Array<string>;
	JSONSDK_FETCH_API_URL: string;
	JSONSDK_FETCH_API_KEY: string;
	JSONSDK_FETCH_API_SECRET: string;
	OIDC_MICROSOFT: boolean;
	OIDC_MICROSOFT_ISSUER_URL: string;
	OIDC_MICROSOFT_CLIENT_ID: string;
	OIDC_MICROSOFT_CLIENT_SECRET: string;
	OIDC_MICROSOFT_SCOPES: Array<string>;
	OIDC_GOOGLE: boolean;
	OIDC_GOOGLE_ISSUER_URL: string;
	OIDC_GOOGLE_CLIENT_ID: string;
	OIDC_GOOGLE_CLIENT_SECRET: string;
	OIDC_GOOGLE_SCOPES: Array<string>;
	OIDC_GITLAB: boolean;
	OIDC_GITLAB_ISSUER_URL: string;
	OIDC_GITLAB_CLIENT_ID: string;
	OIDC_GITLAB_CLIENT_SECRET: string;
	OIDC_GITLAB_SCOPES: Array<string>;
	OIDC_GITHUB: boolean;
	OIDC_GITHUB_ISSUER_URL: string;
	OIDC_GITHUB_CLIENT_ID: string;
	OIDC_GITHUB_CLIENT_SECRET: string;
	OIDC_GITHUB_SCOPES: Array<string>;
	OIDC_WEIBO: boolean;
	OIDC_WEIBO_ISSUER_URL: string;
	OIDC_WEIBO_CLIENT_ID: string;
	OIDC_WEIBO_CLIENT_SECRET: string;
	OIDC_WEIBO_SCOPES: Array<string>;
	CUSTOM_WECHAT_APP_NAME: string;
	CUSTOM_WECHAT_APP_ID: string;
	CUSTOM_WECHAT_APP_SECRET: string;
	CUSTOM_GOOGLE_API_KEY: string;
	CUSTOM_GOOGLE_ANALYTICS_ID: string;
	CUSTOM_GOOGLE_API_EMAIL_ID: string;
	CUSTOM_GOOGLE_API_PRIVATE_KEY: string;
	CUSTOM_AWS_ACCESS_KEY_ID: string;
	CUSTOM_AWS_SECRET_ACCESS_KEY: string;
	CUSTOM_AWS_REGION: string;
	CUSTOM_STRIPE_PUBLIC_KEY: string;
	CUSTOM_STRIPE_SECRET_KEY: string;
	CUSTOM_MAILGUN_API_KEY: string;
	CUSTOM_KUBERNETES_API_URL: string;
	CUSTOM_KUBERNETES_API_TOKEN: string;
	TELEGRAM_BOT_ID: string;
	TELEGRAM_BOT_TOKEN: string;
	TELEGRAM_BOT_SECRET_KEY: string;
}
export declare abstract class WebFoundationApplication<S extends WebFoundationSettings> extends WebApplication<S> {
	protected middleware: Sequence;
	addMiddleware(step: StepFunction): void;
	invoke(ctx: WebContext<S>): Promise<any>;
	protected onInit(): void;
}
export interface WfReq extends WebFoundationRequest<WebFoundationSettings> {
}
declare class WebFoundationRequestIdentity extends WebRequestIdentity {
	session?: string;
	user?: string;
	token?: string;
	roles?: Array<string>;
	groups?: Array<string>;
	scope?: Array<string>;
	profile?: IProfile;
	timezone: string;
	locale: string;
	theme: string;
	static [ClassInitializer](req: WfReq, parameters: Array<any>): WebFoundationRequestIdentity;
}
export interface WebFoundationRequest<S extends WebFoundationSettings> extends WebRequest<S> {
	readonly identity: WebFoundationRequestIdentity;
}
export interface WebFoundationContext<S extends WebFoundationSettings> extends WebContext<S> {
	readonly req: WebFoundationRequest<S>;
	readonly identity: WebFoundationRequestIdentity;
}
export interface Attachment {
	name: string;
	data: Buffer;
	originalname: string;
	mimetype: string;
	encoding: string;
}
export declare abstract class FormField {
	name: string;
}
export declare class FileField extends FormField {
	name: string;
	data: Buffer;
	originalname: string;
	mimetype: string;
	encoding: string;
}
export declare class TextField extends FormField {
	name: string;
	value: string;
}
export declare class WebForm extends Array<FormField> {
	private texts;
	private files;
	private prepare;
	getText(name: string): string | undefined;
	getFile(name?: string): Attachment | undefined;
	getFiles(name?: string): Array<Attachment>;
}
declare class WebFoundationService extends Resolvable<WebFoundationSettings> {
}
export interface Ctx extends Context<WebFoundationSettings> {
}
export declare type ProviderProtocols = string | 'openid' | 'oauth' | 'oauth2' | 'bot';
export interface IExternalProviderPrimaryKey {
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	appID: string;
}
export interface IExternalProvider extends IExternalProviderPrimaryKey, IProfile {
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	providerFingerprint?: string;
	providerConfiguration?: any;
	appID: string;
	appSecret: string;
	scope: string;
	displayName: string;
	displayImageURL: string;
	description: string;
}
declare class ExternalProviderModel extends Model implements IExternalProvider {
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	providerFingerprint: string;
	providerConfiguration: any;
	appID: string;
	appSecret: string;
	scope: string;
	displayName: string;
	displayImageURL: string;
	description: string;
	static toJSON(a: any): any;
	fromJSON(json: any): void;
}
export declare class ExternalProviderService extends WebFoundationService {
	findOrCreateExternalProvider(ctx: Ctx, profile: any): Promise<ExternalProviderModel>;
}
export interface IExternalUserKeyFields {
	uid: number | string;
	providerId: string;
}
export interface IExternalUserMandatoryFields extends IExternalUserKeyFields {
	uid: number | string;
	provider: string;
	providerProtocol: string;
	providerURL: string;
	providerId: string;
	providerData: any;
	userData: any;
}
export interface IExternalUserOptionalFields {
	user?: string;
	username: string | null;
	domain: string | null;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string | null;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	locale: string | null;
}
export interface IExternalUser extends IExternalUserMandatoryFields, IExternalUserOptionalFields {
	_id: ObjectId;
	uid: number | string;
	provider: string;
	providerProtocol: string;
	providerURL: string;
	providerId: string;
	providerData: string;
	userData: string;
	user?: string;
	username: string | null;
	domain: string | null;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string | null;
	displayImageFile: string | null;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	locale: string | null;
	expireAt: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IProfile;
	updatedAt?: Date;
	updatedFrom?: string;
	updatedBy?: IProfile;
	blocked?: boolean;
	blockedAt?: Date;
	blockedFrom?: string;
	blockedBy?: IProfile;
	blockedReason?: string;
	deleted?: boolean;
	deletedAt?: Date;
	deletedFrom?: string;
	deletedBy?: IProfile;
	deletedReason?: string;
}
declare class ExternalUserModel extends Model implements IExternalUser {
	uid: number | string;
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	providerId: string;
	providerData: string;
	userData: string;
	user: string;
	username: string;
	domain: string;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string;
	displayImageFile: string;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	locale: string | null;
}
export declare class ExternalUserService extends WebFoundationService {
	findOrCreateExternalUser(ctx: Ctx, id: PrimaryKey, profile: any): Promise<ExternalUserModel>;
}
export interface WfCtx extends WebFoundationContext<WebFoundationSettings> {
}
declare class WebFoundationController extends Resolvable<WebFoundationSettings> {
}
export declare class TelegramBotController extends WebFoundationController {
	private bot;
	telegram(ctx: WfCtx): Promise<void>;
}
export interface MessageProfile extends IProfile {
	_id: string;
	id: string;
	bot?: boolean;
}
declare class MessageIdentity extends WebFoundationRequestIdentity {
	user: string;
	profile: MessageProfile;
}
export interface BotEvent {
	id: string;
	type: string;
	identity: MessageIdentity;
	timestamp: Date;
	from: {
		id: string | number;
		name: string;
	};
	conversation: {
		id: string;
		isGroup: boolean;
	};
	recipient: {
		id: string | number;
		name: string;
	};
	channelId: string;
	channelData: any;
	text: string;
}
export declare class BotService extends WebFoundationService {
	dispatch(ctx: Context<WebFoundationSettings>, event: BotEvent): void;
}